@@ -1,9 +1,10 @@
 from gym.envs.registration import register
 
+
 register(
     id='jiminy-cartpole-v0',
     entry_point='gym_jiminy.envs:JiminyCartPoleEnv',
-    reward_threshold=10000.0,
+    reward_threshold=10000.0
 )
 
 register(
@@ -13,4 +14,5 @@ register(
     reward_threshold=-3000.0
 )
 
+
 import jiminy_py # Preload jiminy to force import of eigenpy and pinocchio
